# -*- coding: utf-8

from xmlrunner.unittest import unittest

import xml.etree.ElementTree as ET
from xml.dom.minidom import Document

from xmlrunner import builder
import six


class TestXMLContextTest(unittest.TestCase):
    """TestXMLContext test cases.
    """

    def setUp(self):
        self.doc = Document()
        self.root = builder.TestXMLContext(self.doc)

    def test_current_element_tag_name(self):
        self.root.begin('tag', 'context-name')
        self.assertEqual(self.root.element_tag(), 'tag')

    def test_current_context_name(self):
        self.root.begin('tag', 'context-name')
        name = self.root.element.getAttribute('name')
        self.assertEqual(name, 'context-name')

    def test_current_context_invalid_unicode_name(self):
        self.root.begin('tag', six.u('context-name\x01\x0B'))
        name = self.root.element.getAttribute('name')
        self.assertEqual(name, six.u('context-name\uFFFD\uFFFD'))

    def test_increment_valid_testsuites_counters(self):
        self.root.begin('testsuites', 'name')

        for c in ('tests', 'failures', 'errors', 'skipped'):
            self.root.increment_counter(c)

        element = self.root.end()

        with self.assertRaises(KeyError):
            element.attributes['skipped']

        for c in ('tests', 'failures', 'errors'):
            value = element.attributes[c].value
            self.assertEqual(value, '1')

    def test_increment_valid_testsuite_counters(self):
        self.root.begin('testsuite', 'name')

        for c in ('tests', 'failures', 'errors', 'skipped'):
            self.root.increment_counter(c)

        element = self.root.end()

        for c in ('tests', 'failures', 'errors', 'skipped'):
            value = element.attributes[c].value
            self.assertEqual(value, '1')

    def test_increment_counters_for_unknown_context(self):
        self.root.begin('unknown', 'name')

        for c in ('tests', 'failures', 'errors', 'skipped', 'invalid'):
            self.root.increment_counter(c)

        element = self.root.end()

        for c in ('tests', 'failures', 'errors', 'skipped', 'invalid'):
            with self.assertRaises(KeyError):
                element.attributes[c]

    def test_empty_counters_on_end_context(self):
        self.root.begin('testsuite', 'name')
        element = self.root.end()

        for c in ('tests', 'failures', 'errors', 'skipped'):
            self.assertEqual(element.attributes[c].value, '0')

    def test_add_time_attribute_on_end_context(self):
        self.root.begin('testsuite', 'name')
        element = self.root.end()

        element.attributes['time'].value


class TestXMLBuilderTest(unittest.TestCase):
    """TestXMLBuilder test cases.
    """

    def setUp(self):
        self.builder = builder.TestXMLBuilder()
        self.doc = self.builder._xml_doc
        self.builder.begin_context('testsuites', 'name')

        self.valid_chars = six.u('выбор')

        self.invalid_chars = '\x01'
        self.invalid_chars_replace = six.u('\ufffd')

    def test_root_has_no_parent(self):
        self.assertIsNone(self.builder.current_context().parent)

    def test_current_context_tag(self):
        self.assertEqual(self.builder.context_tag(), 'testsuites')

    def test_begin_nested_context(self):
        root = self.builder.current_context()

        self.builder.begin_context('testsuite', 'name')

        self.assertEqual(self.builder.context_tag(), 'testsuite')
        self.assertIs(self.builder.current_context().parent, root)

    def test_end_inexistent_context(self):
        self.builder = builder.TestXMLBuilder()

        self.assertFalse(self.builder.end_context())
        self.assertEqual(len(self.doc.childNodes), 0)

    def test_end_root_context(self):
        root = self.builder.current_context()

        self.assertTrue(self.builder.end_context())
        self.assertIsNone(self.builder.current_context())

        # No contexts left
        self.assertFalse(self.builder.end_context())

        doc_children = self.doc.childNodes

        self.assertEqual(len(doc_children), 1)
        self.assertEqual(len(doc_children[0].childNodes), 0)
        self.assertEqual(doc_children[0].tagName, root.element_tag())

    def test_end_nested_context(self):
        self.builder.begin_context('testsuite', 'name')
        nested = self.builder.current_context()

        self.assertTrue(self.builder.end_context())

        # Only updates the document when all contexts end
        self.assertEqual(len(self.doc.childNodes), 0)

    def test_end_all_context_stack(self):
        root = self.builder.current_context()

        self.builder.begin_context('testsuite', 'name')
        nested = self.builder.current_context()

        self.assertTrue(self.builder.end_context())
        self.assertTrue(self.builder.end_context())

        # No contexts left
        self.assertFalse(self.builder.end_context())

        root_child = self.doc.childNodes

        self.assertEqual(len(root_child), 1)
        self.assertEqual(root_child[0].tagName, root.element_tag())

        nested_child = root_child[0].childNodes

        self.assertEqual(len(nested_child), 1)
        self.assertEqual(nested_child[0].tagName, nested.element_tag())

    def test_append_valid_unicode_cdata_section(self):
        self.builder.append_cdata_section('tag', self.valid_chars)
        self.builder.end_context()

        root_child = self.doc.childNodes[0]

        cdata_container = root_child.childNodes[0]
        self.assertEqual(cdata_container.tagName, 'tag')

        cdata = cdata_container.childNodes[0]
        self.assertEqual(cdata.data, self.valid_chars)

    def test_append_invalid_unicode_cdata_section(self):
        self.builder.append_cdata_section('tag', self.invalid_chars)
        self.builder.end_context()

        root_child = self.doc.childNodes[0]
        cdata_container = root_child.childNodes[0]

        cdata = cdata_container.childNodes[0]
        self.assertEqual(cdata.data, self.invalid_chars_replace)

    def test_append_cdata_closing_tags_into_cdata_section(self):
        self.builder.append_cdata_section('tag',']]>')
        self.builder.end_context()
        root_child = self.doc.childNodes[0]
        cdata_container = root_child.childNodes[0]
        self.assertEqual(len(cdata_container.childNodes), 2)
        self.assertEqual(cdata_container.childNodes[0].data, ']]')
        self.assertEqual(cdata_container.childNodes[1].data, '>')

    def test_append_tag_with_valid_unicode_values(self):
        self.builder.append('tag', self.valid_chars, attr=self.valid_chars)
        self.builder.end_context()

        root_child = self.doc.childNodes[0]
        tag = root_child.childNodes[0]

        self.assertEqual(tag.tagName, 'tag')
        self.assertEqual(tag.getAttribute('attr'), self.valid_chars)
        self.assertEqual(tag.childNodes[0].data, self.valid_chars)

    def test_append_tag_with_invalid_unicode_values(self):
        self.builder.append('tag', self.invalid_chars, attr=self.invalid_chars)
        self.builder.end_context()

        root_child = self.doc.childNodes[0]
        tag = root_child.childNodes[0]

        self.assertEqual(tag.tagName, 'tag')
        self.assertEqual(tag.getAttribute('attr'), self.invalid_chars_replace)
        self.assertEqual(tag.childNodes[0].data, self.invalid_chars_replace)

    def test_increment_root_context_counter(self):
        self.builder.increment_counter('tests')
        self.builder.end_context()

        root_child = self.doc.childNodes[0]

        self.assertEqual(root_child.tagName, 'testsuites')
        self.assertEqual(root_child.getAttribute('tests'), '1')

    def test_increment_nested_context_counter(self):
        self.builder.increment_counter('tests')

        self.builder.begin_context('testsuite', 'name')
        self.builder.increment_counter('tests')

        self.builder.end_context()
        self.builder.end_context()

        root_child = self.doc.childNodes[0]
        nested_child = root_child.childNodes[0]

        self.assertEqual(root_child.tagName, 'testsuites')
        self.assertEqual(nested_child.getAttribute('tests'), '1')
        self.assertEqual(root_child.getAttribute('tests'), '2')

    def test_finish_nested_context(self):
        self.builder.begin_context('testsuite', 'name')

        tree = ET.fromstring(self.builder.finish())

        self.assertEqual(tree.tag, 'testsuites')
        self.assertEqual(len(tree.findall("./testsuite")), 1)




import os
import sys
import time
import six
import re
from os import path
from six.moves import StringIO

from .unittest import TestResult, _TextTestResult


# Matches invalid XML1.0 unicode characters, like control characters:
# http://www.w3.org/TR/2006/REC-xml-20060816/#charsets
INVALID_XML_1_0_UNICODE_RE = re.compile(
    u'[\x00-\x08\x0B\x0C\x0E-\x1F\uD800-\uDFFF\uFFFE\uFFFF]',
    re.UNICODE
)


def xml_safe_unicode(base, encoding='utf-8'):
    """Return a unicode string containing only valid XML characters.

    encoding - if base is a byte string it is first decoded to unicode
        using this encoding.
    """
    if isinstance(base, six.binary_type):
        base = base.decode(encoding)
    return INVALID_XML_1_0_UNICODE_RE.sub('', base)


def to_unicode(data):
    """Returns unicode in Python2 and str in Python3"""
    if six.PY3:
        return six.text_type(data)
    try:
        # Try utf8
        return six.text_type(data)
    except UnicodeDecodeError:
        return repr(data).decode('utf8', 'replace')


def safe_unicode(data, encoding=None):
    return xml_safe_unicode(to_unicode(data), encoding)


def testcase_name(test_method):
    testcase = type(test_method)

    # Ignore module name if it is '__main__'
    module = testcase.__module__ + '.'
    if module == '__main__.':
        module = ''
    result = module + testcase.__name__
    return result


class _TestInfo(object):
    """
    This class keeps useful information about the execution of a
    test method.
    """

    # Possible test outcomes
    (SUCCESS, FAILURE, ERROR, SKIP) = range(4)

    def __init__(self, test_result, test_method, outcome=SUCCESS, err=None):
        self.test_result = test_result
        self.outcome = outcome
        self.elapsed_time = 0
        self.err = err
        self.stdout = test_result._stdout_data
        self.stderr = test_result._stderr_data

        self.test_description = self.test_result.getDescription(test_method)
        self.test_exception_info = (
            '' if outcome in (self.SUCCESS, self.SKIP)
            else self.test_result._exc_info_to_string(
                    self.err, test_method)
        )

        self.test_name = testcase_name(test_method)
        self.test_id = test_method.id()

    def id(self):
        return self.test_id

    def test_finished(self):
        """Save info that can only be calculated once a test has run.
        """
        self.elapsed_time = \
            self.test_result.stop_time - self.test_result.start_time

    def get_description(self):
        """
        Return a text representation of the test method.
        """
        return self.test_description

    def get_error_info(self):
        """
        Return a text representation of an exception thrown by a test
        method.
        """
        return self.test_exception_info


class _XMLTestResult(_TextTestResult):
    """
    A test result class that can express test results in a XML report.

    Used by XMLTestRunner.
    """
    def __init__(self, stream=sys.stderr, descriptions=1, verbosity=1,
                 elapsed_times=True, properties=None):
        _TextTestResult.__init__(self, stream, descriptions, verbosity)
        self.buffer = True  # we are capturing test output
        self._stdout_data = None
        self._stderr_data = None
        self.successes = []
        self.callback = None
        self.elapsed_times = elapsed_times
        self.properties = None  # junit testsuite properties

    def _prepare_callback(self, test_info, target_list, verbose_str,
                          short_str):
        """
        Appends a _TestInfo to the given target list and sets a callback
        method to be called by stopTest method.
        """
        target_list.append(test_info)

        def callback():
            """Prints the test method outcome to the stream, as well as
            the elapsed time.
            """

            test_info.test_finished()

            # Ignore the elapsed times for a more reliable unit testing
            if not self.elapsed_times:
                self.start_time = self.stop_time = 0

            if self.showAll:
                self.stream.writeln(
                    '%s (%.3fs)' % (verbose_str, test_info.elapsed_time)
                )
            elif self.dots:
                self.stream.write(short_str)
        self.callback = callback

    def startTest(self, test):
        """
        Called before execute each test method.
        """
        self.start_time = time.time()
        TestResult.startTest(self, test)

        if self.showAll:
            self.stream.write('  ' + self.getDescription(test))
            self.stream.write(" ... ")

    def _save_output_data(self):
        self._stdout_data = sys.stdout.getvalue()
        self._stderr_data = sys.stderr.getvalue()

    def stopTest(self, test):
        """
        Called after execute each test method.
        """
        self._save_output_data()
        # self._stdout_data = sys.stdout.getvalue()
        # self._stderr_data = sys.stderr.getvalue()

        _TextTestResult.stopTest(self, test)
        self.stop_time = time.time()

        if self.callback and callable(self.callback):
            self.callback()
            self.callback = None

    def addSuccess(self, test):
        """
        Called when a test executes successfully.
        """
        self._save_output_data()
        self._prepare_callback(
            _TestInfo(self, test), self.successes, 'OK', '.'
        )

    def addFailure(self, test, err):
        """
        Called when a test method fails.
        """
        self._save_output_data()
        testinfo = _TestInfo(self, test, _TestInfo.FAILURE, err)
        self.failures.append((
            testinfo,
            self._exc_info_to_string(err, test)
        ))
        self._prepare_callback(testinfo, [], 'FAIL', 'F')

    def addError(self, test, err):
        """
        Called when a test method raises an error.
        """
        self._save_output_data()
        testinfo = _TestInfo(self, test, _TestInfo.ERROR, err)
        self.errors.append((
            testinfo,
            self._exc_info_to_string(err, test)
        ))
        self._prepare_callback(testinfo, [], 'ERROR', 'E')

    def addSkip(self, test, reason):
        """
        Called when a test method was skipped.
        """
        self._save_output_data()
        testinfo = _TestInfo(self, test, _TestInfo.SKIP, reason)
        self.skipped.append((testinfo, reason))
        self._prepare_callback(testinfo, [], 'SKIP', 'S')

    def printErrorList(self, flavour, errors):
        """
        Writes information about the FAIL or ERROR to the stream.
        """
        for test_info, error in errors:
            self.stream.writeln(self.separator1)
            self.stream.writeln(
                '%s [%.3fs]: %s' % (flavour, test_info.elapsed_time,
                                    test_info.get_description())
            )
            self.stream.writeln(self.separator2)
            self.stream.writeln('%s' % test_info.get_error_info())

    def _get_info_by_testcase(self):
        """
        Organizes test results by TestCase module. This information is
        used during the report generation, where a XML report will be created
        for each TestCase.
        """
        tests_by_testcase = {}

        for tests in (self.successes, self.failures, self.errors,
                      self.skipped):
            for test_info in tests:
                if isinstance(test_info, tuple):
                    # This is a skipped, error or a failure test case
                    test_info = test_info[0]
                testcase_name = test_info.test_name
                if testcase_name not in tests_by_testcase:
                    tests_by_testcase[testcase_name] = []
                tests_by_testcase[testcase_name].append(test_info)

        return tests_by_testcase

    def _report_testsuite_properties(xml_testsuite, xml_document, properties):
        xml_properties = xml_document.createElement('properties')
        xml_testsuite.appendChild(xml_properties)
        if properties:
            for key, value in properties.items():
                prop = xml_document.createElement('property')
                prop.setAttribute('name', str(key))
                prop.setAttribute('value', str(value))
                xml_properties.appendChild(prop)
        return xml_properties

    _report_testsuite_properties = staticmethod(_report_testsuite_properties)

    def _report_testsuite(suite_name, tests, xml_document, parentElement,
                          properties):
        """
        Appends the testsuite section to the XML document.
        """
        testsuite = xml_document.createElement('testsuite')
        parentElement.appendChild(testsuite)

        testsuite.setAttribute('name', suite_name)
        testsuite.setAttribute('tests', str(len(tests)))

        testsuite.setAttribute(
            'time', '%.3f' % sum(map(lambda e: e.elapsed_time, tests))
        )
        failures = filter(lambda e: e.outcome == _TestInfo.FAILURE, tests)
        testsuite.setAttribute('failures', str(len(list(failures))))

        errors = filter(lambda e: e.outcome == _TestInfo.ERROR, tests)
        testsuite.setAttribute('errors', str(len(list(errors))))

        _XMLTestResult._report_testsuite_properties(
            testsuite, xml_document, properties)

        systemout = xml_document.createElement('system-out')
        testsuite.appendChild(systemout)

        stdout = StringIO()
        for test in tests:
            # Merge the stdout from the tests in a class
            stdout.write(test.stdout)
        _XMLTestResult._createCDATAsections(
            xml_document, systemout, stdout.getvalue())

        systemerr = xml_document.createElement('system-err')
        testsuite.appendChild(systemerr)

        stderr = StringIO()
        for test in tests:
            # Merge the stderr from the tests in a class
            stderr.write(test.stderr)
        _XMLTestResult._createCDATAsections(
            xml_document, systemerr, stderr.getvalue())

        return testsuite

    _report_testsuite = staticmethod(_report_testsuite)

    def _test_method_name(test_id):
        """
        Returns the test method name.
        """
        return test_id.split('.')[-1]

    _test_method_name = staticmethod(_test_method_name)

    def _createCDATAsections(xmldoc, node, text):
        text = safe_unicode(text)
        pos = text.find(']]>')
        while pos >= 0:
            tmp = text[0:pos+2]
            cdata = xmldoc.createCDATASection(tmp)
            node.appendChild(cdata)
            text = text[pos+2:]
            pos = text.find(']]>')
        cdata = xmldoc.createCDATASection(text)
        node.appendChild(cdata)

    _createCDATAsections = staticmethod(_createCDATAsections)

    def _report_testcase(suite_name, test_result, xml_testsuite, xml_document):
        """
        Appends a testcase section to the XML document.
        """
        testcase = xml_document.createElement('testcase')
        xml_testsuite.appendChild(testcase)

        testcase.setAttribute('classname', suite_name)
        testcase.setAttribute(
            'name', _XMLTestResult._test_method_name(test_result.test_id)
        )
        testcase.setAttribute('time', '%.3f' % test_result.elapsed_time)

        if (test_result.outcome != _TestInfo.SUCCESS):
            elem_name = ('failure', 'error', 'skipped')[test_result.outcome-1]
            failure = xml_document.createElement(elem_name)
            testcase.appendChild(failure)
            if test_result.outcome != _TestInfo.SKIP:
                failure.setAttribute(
                    'type',
                    safe_unicode(test_result.err[0].__name__)
                )
                failure.setAttribute(
                    'message',
                    safe_unicode(test_result.err[1])
                )
                error_info = safe_unicode(test_result.get_error_info())
                _XMLTestResult._createCDATAsections(
                    xml_document, failure, error_info)
            else:
                failure.setAttribute('type', 'skip')
                failure.setAttribute('message', safe_unicode(test_result.err))

    _report_testcase = staticmethod(_report_testcase)

    def generate_reports(self, test_runner):
        """
        Generates the XML reports to a given XMLTestRunner object.
        """
        from xml.dom.minidom import Document
        all_results = self._get_info_by_testcase()

        outputHandledAsString = \
            isinstance(test_runner.output, six.string_types)

        if (outputHandledAsString and not os.path.exists(test_runner.output)):
            os.makedirs(test_runner.output)

        if not outputHandledAsString:
            doc = Document()
            testsuite = doc.createElement('testsuites')
            doc.appendChild(testsuite)
            parentElement = testsuite

        for suite, tests in all_results.items():
            if outputHandledAsString:
                doc = Document()
                parentElement = doc

            suite_name = suite
            if test_runner.outsuffix:
                # not checking with 'is not None', empty means no suffix.
                suite_name = '%s-%s' % (suite, test_runner.outsuffix)

            # Build the XML file
            testsuite = _XMLTestResult._report_testsuite(
                suite_name, tests, doc, parentElement, self.properties
            )
            for test in tests:
                _XMLTestResult._report_testcase(suite, test, testsuite, doc)
            xml_content = doc.toprettyxml(
                indent='\t',
                encoding=test_runner.encoding
            )

            if outputHandledAsString:
                filename = path.join(
                    test_runner.output,
                    'TEST-%s.xml' % suite_name)
                with open(filename, 'wb') as report_file:
                    report_file.write(xml_content)

        if not outputHandledAsString:
            # Assume that test_runner.output is a stream
            test_runner.output.write(xml_content)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Executable module to test unittest-xml-reporting.
"""
import sys

from xmlrunner.unittest import unittest
import xmlrunner
from six import StringIO, BytesIO
from tempfile import mkdtemp
from shutil import rmtree
from glob import glob
from xml.dom import minidom
import os.path


class XMLTestRunnerTestCase(unittest.TestCase):
    """
    XMLTestRunner test case.
    """
    class DummyTest(unittest.TestCase):
        @unittest.skip("demonstrating skipping")
        def test_skip(self):
            pass   # pragma: no cover
        @unittest.skip(u"demonstrating non-ascii skipping: éçà")
        def test_non_ascii_skip(self):
            pass   # pragma: no cover
        def test_pass(self):
            pass
        def test_fail(self):
            self.assertTrue(False)
        @unittest.expectedFailure
        def test_expected_failure(self):
            self.assertTrue(False)
        @unittest.expectedFailure
        def test_unexpected_success(self):
            pass
        def test_error(self):
            1 / 0
        def test_cdata_section(self):
            print('<![CDATA[content]]>')
        def test_non_ascii_error(self):
            self.assertEqual(u"éçà", 42)
        def test_unsafe_unicode(self):
            print(u"A\x00B\x08C\x0BD\x0C")
        def test_runner_buffer_output_pass(self):
            print('should not be printed')
        def test_runner_buffer_output_fail(self):
            print('should be printed')
            self.fail('expected to fail')

    def setUp(self):
        self.stream = StringIO()
        self.outdir = mkdtemp()
        self.verbosity = 0
        self.runner_kwargs = {}
        self.addCleanup(rmtree, self.outdir)

    def _test_xmlrunner(self, suite, runner=None):
        outdir = self.outdir
        stream = self.stream
        verbosity = self.verbosity
        runner_kwargs = self.runner_kwargs
        if runner is None:
            runner = xmlrunner.XMLTestRunner(
                stream=stream, output=outdir, verbosity=verbosity,
                **self.runner_kwargs)
        self.assertEqual(0, len(glob(os.path.join(outdir, '*xml'))))
        runner.run(suite)
        self.assertEqual(1, len(glob(os.path.join(outdir, '*xml'))))
        return runner

    def test_basic_unittest_constructs(self):
        suite = unittest.TestSuite()
        suite.addTest(self.DummyTest('test_pass'))
        suite.addTest(self.DummyTest('test_skip'))
        suite.addTest(self.DummyTest('test_fail'))
        suite.addTest(self.DummyTest('test_expected_failure'))
        suite.addTest(self.DummyTest('test_unexpected_success'))
        suite.addTest(self.DummyTest('test_error'))
        self._test_xmlrunner(suite)

    def test_xmlrunner_non_ascii(self):
        suite = unittest.TestSuite()
        suite.addTest(self.DummyTest('test_non_ascii_skip'))
        suite.addTest(self.DummyTest('test_non_ascii_error'))
        outdir = BytesIO()
        runner = xmlrunner.XMLTestRunner(
            stream=self.stream, output=outdir, verbosity=self.verbosity,
            **self.runner_kwargs)
        runner.run(suite)
        outdir.seek(0)
        output = outdir.read()
        self.assertIn(
            u'<skipped message="demonstrating non-ascii skipping: éçà" type="skip"/>'.encode('utf8'),
            output)

    def test_xmlrunner_safe_xml_encoding_name(self):
        suite = unittest.TestSuite()
        suite.addTest(self.DummyTest('test_pass'))
        outdir = BytesIO()
        runner = xmlrunner.XMLTestRunner(
            stream=self.stream, output=outdir, verbosity=self.verbosity,
            **self.runner_kwargs)
        runner.run(suite)
        outdir.seek(0)
        output = outdir.read()
        firstline = output.splitlines()[0]
        # test for issue #74
        self.assertIn('encoding="UTF-8"'.encode('utf8'), firstline)
        
    def test_xmlrunner_check_for_valid_xml_streamout(self):
        """
        This test checks if the xml document is valid if there are more than
        one testsuite and the output of the report is a single stream.
        """
        class DummyTestA(unittest.TestCase):
            def test_pass(self):
                pass
        class DummyTestB(unittest.TestCase):
            def test_pass(self):
                pass
        suite = unittest.TestSuite()
        suite.addTest( unittest.TestLoader().loadTestsFromTestCase(DummyTestA) );
        suite.addTest( unittest.TestLoader().loadTestsFromTestCase(DummyTestB) );
        outdir = BytesIO()
        runner = xmlrunner.XMLTestRunner(
            stream=self.stream, output=outdir, verbosity=self.verbosity,
            **self.runner_kwargs)
        runner.run(suite)
        outdir.seek(0)
        output = outdir.read()
        # Finally check if we have a valid XML document or not.        
        try:
            minidom.parseString(output)
        except Exception as e:
            self.fail(e)

    def test_xmlrunner_unsafe_unicode(self):
        suite = unittest.TestSuite()
        suite.addTest(self.DummyTest('test_unsafe_unicode'))
        outdir = BytesIO()
        runner = xmlrunner.XMLTestRunner(
            stream=self.stream, output=outdir, verbosity=self.verbosity,
            **self.runner_kwargs)
        runner.run(suite)
        outdir.seek(0)
        output = outdir.read()
        self.assertIn(u"<![CDATA[ABCD\n]]>".encode('utf8'), output)

    @unittest.expectedFailure
    def test_xmlrunner_buffer_output_pass(self):
        suite = unittest.TestSuite()
        suite.addTest(self.DummyTest('test_runner_buffer_output_pass'))
        self._test_xmlrunner(suite)
        testsuite_output = self.stream.getvalue()
        # Since we are always buffering stdout/stderr
        # it is currently troublesome to print anything at all
        # and be consistent with --buffer option (issue #59)
        self.assertIn('should not be printed', testsuite_output)
        # this will be fixed when using the composite approach
        # that was under development in the rewrite branch.

    def test_xmlrunner_buffer_output_fail(self):
        suite = unittest.TestSuite()
        suite.addTest(self.DummyTest('test_runner_buffer_output_fail'))
        self._test_xmlrunner(suite)
        testsuite_output = self.stream.getvalue()
        self.assertIn('should be printed', testsuite_output)

    def test_xmlrunner_pass(self):
        suite = unittest.TestSuite()
        suite.addTest(self.DummyTest('test_pass'))
        self._test_xmlrunner(suite)

    def test_xmlrunner_verbose(self):
        self.verbosity = 1
        suite = unittest.TestSuite()
        suite.addTest(self.DummyTest('test_pass'))
        self._test_xmlrunner(suite)

    def test_xmlrunner_showall(self):
        self.verbosity = 2
        suite = unittest.TestSuite()
        suite.addTest(self.DummyTest('test_pass'))
        self._test_xmlrunner(suite)

    def test_xmlrunner_cdata_section(self):
        suite = unittest.TestSuite()
        suite.addTest(self.DummyTest('test_cdata_section'))
        self._test_xmlrunner(suite)

    def test_xmlrunner_outsuffix(self):
        self.runner_kwargs['outsuffix'] = '.somesuffix'
        suite = unittest.TestSuite()
        suite.addTest(self.DummyTest('test_pass'))
        self._test_xmlrunner(suite)
        xmlfile = glob(os.path.join(self.outdir, '*xml'))[0]
        assert xmlfile.endswith('.somesuffix.xml')

    def test_xmlrunner_nosuffix(self):
        self.runner_kwargs['outsuffix'] = ''
        suite = unittest.TestSuite()
        suite.addTest(self.DummyTest('test_pass'))
        self._test_xmlrunner(suite)
        xmlfile = glob(os.path.join(self.outdir, '*xml'))[0]
        xmlfile = os.path.basename(xmlfile)
        assert xmlfile.endswith('DummyTest.xml')

    def test_junitxml_properties(self):
        suite = unittest.TestSuite()
        suite.addTest(self.DummyTest('test_pass'))
        suite.properties = dict(key='value')
        self._test_xmlrunner(suite)

    def test_xmlrunner_elapsed_times(self):
        self.runner_kwargs['elapsed_times'] = False
        suite = unittest.TestSuite()
        suite.addTest(self.DummyTest('test_pass'))
        self._test_xmlrunner(suite)

    def test_xmlrunner_stream(self):
        stream = self.stream
        output = BytesIO()
        runner = xmlrunner.XMLTestRunner(
            stream=stream, output=output, verbosity=self.verbosity,
            **self.runner_kwargs)
        suite = unittest.TestSuite()
        suite.addTest(self.DummyTest('test_pass'))
        runner.run(suite)

    def test_xmlrunner_output_subdir(self):
        stream = self.stream
        output = os.path.join(self.outdir, 'subdir')
        runner = xmlrunner.XMLTestRunner(
            stream=stream, output=output, verbosity=self.verbosity,
            **self.runner_kwargs)
        suite = unittest.TestSuite()
        suite.addTest(self.DummyTest('test_pass'))
        runner.run(suite)

    def test_xmlrunner_patched_stdout(self):
        old_stdout, old_stderr = sys.stdout, sys.stderr
        try:
            sys.stdout, sys.stderr = StringIO(), StringIO()
            suite = unittest.TestSuite()
            suite.addTest(self.DummyTest('test_pass'))
            suite.properties = dict(key='value')
            self._test_xmlrunner(suite)
        finally:
            sys.stdout, sys.stderr = old_stdout, old_stderr

